searchData={"items":[{"type":"module","title":"Ethereumex","doc":"Elixir JSON-RPC client for the Ethereum blockchain.\n\nCheck out the documentation [here](https://hexdocs.pm/ethereumex/Ethereumex.html#content).","ref":"Ethereumex.html"},{"type":"module","title":"Installation - Ethereumex","doc":"Add `:ethereumex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ethereumex, \"~> 0.11.1\"}\n  ]\nend\n```","ref":"Ethereumex.html#module-installation"},{"type":"module","title":"Configuration - Ethereumex","doc":"","ref":"Ethereumex.html#module-configuration"},{"type":"module","title":"HTTP - Ethereumex","doc":"In `config/config.exs`, add Ethereum protocol host params to your config file\n\n```elixir\nconfig :ethereumex,\n  url: \"http://localhost:8545\"\n```\n\nYou can also configure the `HTTP` request timeout for requests sent to the Ethereum JSON-RPC\n(you can also overwrite this configuration in `opts` used when calling the client).\n\n```elixir\nconfig :ethereumex,\n  http_options: [pool_timeout: 5000, receive_timeout: 15_000],\n  http_headers: [{\"Content-Type\", \"application/json\"}]\n```\n\n`:pool_timeout` - This timeout is applied when we check out a connection from the pool. Default value is `5_000`.\n`:receive_timeout` - The maximum time to wait for a response before returning an error. Default value is `15_000`","ref":"Ethereumex.html#module-http"},{"type":"module","title":"IPC - Ethereumex","doc":"If you want to use IPC you will need to set a few things in your config.\n\nFirst, specify the `:client_type`:\n\n```elixir\nconfig :ethereumex,\n  client_type: :ipc\n```\n\nThis will resolve to `:http` by default.\n\nSecond, specify the `:ipc_path`:\n\n```elixir\nconfig :ethereumex,\n  ipc_path: \"/path/to/ipc\"\n```\n\nThe IPC client type mode opens a pool of connection workers (default is 5 and 2, respectively). You can configure the pool size.\n\n```elixir\nconfig :ethereumex,\n  ipc_worker_size: 5,\n  ipc_max_worker_overflow: 2,\n  ipc_request_timeout: 60_000\n```","ref":"Ethereumex.html#module-ipc"},{"type":"module","title":"WebSocket - Ethereumex","doc":"The WebSocket client supports both standard JSON-RPC requests and real-time subscriptions.\nTo use it, configure your application with:\n\n```elixir\nconfig :ethereumex,\n  websocket_url: \"ws://localhost:8545\",\n  client_type: :websocket\n```\n\n#### Standard RPC Calls\n\nAll standard RPC methods work the same as with HTTP:\n\n```elixir\niex> Ethereumex.WebsocketClient.eth_block_number()\n{:ok, \"0x1234\"}\n```\n\n#### Real-time Subscriptions\n\nSubscribe to various blockchain events:\n\n```elixir\n# Subscribe to new block headers\niex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:newHeads)\n{:ok, \"0x9cef478923ff08bf67fde6c64013158d\"}\n\n# Subscribe to logs/events from specific contracts\niex> filter = %{\n...>   address: \"0x8320fe7702b96808f7bbc0d4a888ed1468216cfd\",\n...>   topics: [\"0xd78a0cb8bb633d06981248b816e7bd33c2a35a6089241d099fa519e361cab902\"]\n...> }\niex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:logs, filter)\n{:ok, \"0x4a8a4c0517381924f9838102c5a4dcb7\"}\n\n# Subscribe to pending transactions\niex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:newPendingTransactions)\n{:ok, \"0x1234567890abcdef1234567890abcdef\"}\n\n# Receive notifications in your process\nreceive do\n  %{\n    \"method\" => \"eth_subscription\",\n    \"params\" => %{\n      \"subscription\" => subscription_id,\n      \"result\" => result\n    }\n  } -> handle_notification(result)\nend\n\n# Unsubscribe when done\niex> Ethereumex.WebsocketClient.unsubscribe(subscription_id)\n{:ok, true}\n```\n\nAvailable subscription types:\n\n- `:newHeads` - New block headers\n- `:logs` - Contract events/logs with optional filtering\n- `:newPendingTransactions` - Pending transaction hashes","ref":"Ethereumex.html#module-websocket"},{"type":"module","title":"Telemetry - Ethereumex","doc":"If you want to count the number of RPC calls per RPC method or overall,\nyou can attach yourself to executed telemetry events.\nThere are two events you can attach yourself to:\n`[:ethereumex]` # has RPC method name in metadata\nEmitted event: `{:event, [:ethereumex], %{counter: 1}, %{method_name: \"method_name\"}}`\n\nor more granular\n`[:ethereumex,  ]` # %{} metadata\nEmitted event: `{:event, [:ethereumex, :method_name_as_atom], %{counter: 1}, %{}}`\n\nEach event caries a single ticker that you can pass into your counters (like `Statix.increment/2`).\nBe sure to add :telemetry as project dependency.","ref":"Ethereumex.html#module-telemetry"},{"type":"module","title":"Test - Ethereumex","doc":"Download `parity` and initialize the password file\n\n```\n$ make setup\n```\n\nRun `parity`\n\n```\n$ make run\n```\n\nRun tests\n\n```\n$ make test\n```","ref":"Ethereumex.html#module-test"},{"type":"module","title":"Usage - Ethereumex","doc":"","ref":"Ethereumex.html#module-usage"},{"type":"module","title":"Available methods: - Ethereumex","doc":"- [`web3_clientVersion`](https://eth.wiki/json-rpc/API#web3_clientversion)\n- [`web3_sha3`](https://eth.wiki/json-rpc/API#web3_sha3)\n- [`net_version`](https://eth.wiki/json-rpc/API#net_version)\n- [`net_peerCount`](https://eth.wiki/json-rpc/API#net_peercount)\n- [`net_listening`](https://eth.wiki/json-rpc/API#net_listening)\n- [`eth_protocolVersion`](https://eth.wiki/json-rpc/API#eth_protocolversion)\n- [`eth_syncing`](https://eth.wiki/json-rpc/API#eth_syncing)\n- [`eth_coinbase`](https://eth.wiki/json-rpc/API#eth_coinbase)\n- [`eth_chainId`](https://eth.wiki/json-rpc/API#eth_chainId)\n- [`eth_mining`](https://eth.wiki/json-rpc/API#eth_mining)\n- [`eth_hashrate`](https://eth.wiki/json-rpc/API#eth_hashrate)\n- [`eth_gasPrice`](https://eth.wiki/json-rpc/API#eth_gasprice)\n- [`eth_accounts`](https://eth.wiki/json-rpc/API#eth_accounts)\n- [`eth_blockNumber`](https://eth.wiki/json-rpc/API#eth_blocknumber)\n- [`eth_getBalance`](https://eth.wiki/json-rpc/API#eth_getbalance)\n- [`eth_getStorageAt`](https://eth.wiki/json-rpc/API#eth_getstorageat)\n- [`eth_getTransactionCount`](https://eth.wiki/json-rpc/API#eth_gettransactioncount)\n- [`eth_getBlockTransactionCountByHash`](https://eth.wiki/json-rpc/API#eth_getblocktransactioncountbyhash)\n- [`eth_getBlockTransactionCountByNumber`](https://eth.wiki/json-rpc/API#eth_getblocktransactioncountbynumber)\n- [`eth_getUncleCountByBlockHash`](https://eth.wiki/json-rpc/API#eth_getunclecountbyblockhash)\n- [`eth_getUncleCountByBlockNumber`](https://eth.wiki/json-rpc/API#eth_getunclecountbyblocknumber)\n- [`eth_getCode`](https://eth.wiki/json-rpc/API#eth_getcode)\n- [`eth_sign`](https://eth.wiki/json-rpc/API#eth_sign)\n- [`eth_sendTransaction`](https://eth.wiki/json-rpc/API#eth_sendtransaction)\n- [`eth_sendRawTransaction`](https://eth.wiki/json-rpc/API#eth_sendrawtransaction)\n- [`eth_call`](https://eth.wiki/json-rpc/API#eth_call)\n- [`eth_estimateGas`](https://eth.wiki/json-rpc/API#eth_estimategas)\n- [`eth_getBlockByHash`](https://eth.wiki/json-rpc/API#eth_getblockbyhash)\n- [`eth_getBlockByNumber`](https://eth.wiki/json-rpc/API#eth_getblockbynumber)\n- [`eth_getTransactionByHash`](https://eth.wiki/json-rpc/API#eth_gettransactionbyhash)\n- [`eth_getTransactionByBlockHashAndIndex`](https://eth.wiki/json-rpc/API#eth_gettransactionbyblockhashandindex)\n- [`eth_getTransactionByBlockNumberAndIndex`](https://eth.wiki/json-rpc/API#eth_gettransactionbyblocknumberandindex)\n- [`eth_getTransactionReceipt`](https://eth.wiki/json-rpc/API#eth_gettransactionreceipt)\n- [`eth_getUncleByBlockHashAndIndex`](https://eth.wiki/json-rpc/API#eth_getunclebyblockhashandindex)\n- [`eth_getUncleByBlockNumberAndIndex`](https://eth.wiki/json-rpc/API#eth_getunclebyblocknumberandindex)\n- [`eth_getCompilers`](https://eth.wiki/json-rpc/API#eth_getcompilers)\n- [`eth_compileLLL`](https://eth.wiki/json-rpc/API#eth_compilelll)\n- [`eth_compileSolidity`](https://eth.wiki/json-rpc/API#eth_compilesolidity)\n- [`eth_compileSerpent`](https://eth.wiki/json-rpc/API#eth_compileserpent)\n- [`eth_newFilter`](https://eth.wiki/json-rpc/API#eth_newfilter)\n- [`eth_newBlockFilter`](https://eth.wiki/json-rpc/API#eth_newblockfilter)\n- [`eth_newPendingTransactionFilter`](https://eth.wiki/json-rpc/API#eth_newpendingtransactionfilter)\n- [`eth_uninstallFilter`](https://eth.wiki/json-rpc/API#eth_uninstallfilter)\n- [`eth_getFilterChanges`](https://eth.wiki/json-rpc/API#eth_getfilterchanges)\n- [`eth_getFilterLogs`](https://eth.wiki/json-rpc/API#eth_getfilterlogs)\n- [`eth_getLogs`](https://eth.wiki/json-rpc/API#eth_getlogs)\n- eth_getProof\n- [`eth_getWork`](https://eth.wiki/json-rpc/API#eth_getwork)\n- [`eth_submitWork`](https://eth.wiki/json-rpc/API#eth_submitwork)\n- [`eth_submitHashrate`](https://eth.wiki/json-rpc/API#eth_submithashrate)\n- [`db_putString`](https://eth.wiki/json-rpc/API#db_putstring)\n- [`db_getString`](https://eth.wiki/json-rpc/API#db_getstring)\n- [`db_putHex`](https://eth.wiki/json-rpc/API#db_puthex)\n- [`db_getHex`](https://eth.wiki/json-rpc/API#db_gethex)\n- [`shh_post`](https://eth.wiki/json-rpc/API#shh_post)\n- [`shh_version`](https://eth.wiki/json-rpc/API#shh_version)\n- [`shh_newIdentity`](https://eth.wiki/json-rpc/API#shh_newidentity)\n- [`shh_hasIdentity`](https://eth.wiki/json-rpc/API#shh_hasidentity)\n- [`shh_newGroup`](https://eth.wiki/json-rpc/API#shh_newgroup)\n- [`shh_addToGroup`](https://eth.wiki/json-rpc/API#shh_addtogroup)\n- [`shh_newFilter`](https://eth.wiki/json-rpc/API#shh_newfilter)\n- [`shh_uninstallFilter`](https://eth.wiki/json-rpc/API#shh_uninstallfilter)\n- [`shh_getFilterChanges`](https://eth.wiki/json-rpc/API#shh_getfilterchanges)\n- [`shh_getMessages`](https://eth.wiki/json-rpc/API#shh_getmessages)\n\n#### WebSocket Subscription Methods\n\n- [`eth_subscribe`](https://geth.ethereum.org/docs/interacting-with-geth/rpc/pubsub#create-subscriptions) - Subscribe to real-time events\n- [`eth_unsubscribe`](https://geth.ethereum.org/docs/interacting-with-geth/rpc/pubsub#cancel-subscriptions) - Unsubscribe from events","ref":"Ethereumex.html#module-available-methods"},{"type":"module","title":"IpcClient - Ethereumex","doc":"You can follow along with any of these examples using IPC by replacing `HttpClient` with `IpcClient`.","ref":"Ethereumex.html#module-ipcclient"},{"type":"module","title":"Examples - Ethereumex","doc":"```elixir\niex> Ethereumex.HttpClient.web3_client_version\n{:ok, \"Parity//v1.7.2-beta-9f47909-20170918/x86_64-macos/rustc1.19.0\"}\n\n# Using the url option will overwrite the configuration\niex> Ethereumex.HttpClient.web3_client_version(url: \"http://localhost:8545\")\n{:ok, \"Parity//v1.7.2-beta-9f47909-20170918/x86_64-macos/rustc1.19.0\"}\n\niex> Ethereumex.HttpClient.web3_sha3(\"wrong_param\")\n{:error, %{\"code\" => -32602, \"message\" => \"Invalid params: invalid format.\"}}\n\niex> Ethereumex.HttpClient.eth_get_balance(\"0x407d73d8a49eeb85d32cf465507dd71d507100c1\")\n{:ok, \"0x0\"}\n```\n\nNote that all method names are snakecases, so, for example, shh_getMessages method has corresponding Ethereumex.HttpClient.shh_get_messages/1 method. Signatures can be found in Ethereumex.Client.Behaviour. There are more examples in tests.\n\n#### eth_call example - Read only smart contract calls\n\nIn order to call a smart contract using the JSON-RPC interface you need to properly hash the data attribute (this will need to include the contract method signature along with arguments if any). You can do this manually or use a hex package like [ABI](https://hex.pm/packages/ex_abi) to parse your smart contract interface or encode individual calls.\n\n```elixir\ndefp deps do\n  [\n    ...\n    {:ethereumex, \"~> 0.9\"},\n    {:ex_abi, \"~> 0.5\"}\n    ...\n  ]\nend\n```\n\nNow load the ABI and pass the method signature. Note that the address needs to be converted to bytes:\n\n```elixir\naddress           = \"0xF742d4cE7713c54dD701AA9e92101aC42D63F895\" |> String.slice(2..-1) |> Base.decode16!(case: :mixed)\ncontract_address  = \"0xC28980830dD8b9c68a45384f5489ccdAF19D53cC\"\nabi_encoded_data  = ABI.encode(\"balanceOf(address)\", [address]) |> Base.encode16(case: :lower)\n```\n\nNow you can use eth_call to execute this smart contract command:\n\n```elixir\nbalance_bytes = Ethereumex.HttpClient.eth_call(%{\n  data: \"0x\" <> abi_encoded_data,\n  to: contract_address\n})\n```\n\nTo convert the balance into an integer:\n\n```elixir\nbalance_bytes\n|> String.slice(2..-1)\n|> Base.decode16!(case: :lower)\n|> TypeDecoder.decode_raw([{:uint, 256}])\n|> List.first\n```","ref":"Ethereumex.html#module-examples"},{"type":"module","title":"Custom requests - Ethereumex","doc":"Many Ethereum protocol implementations support additional JSON-RPC API methods. To use them, you should call Ethereumex.HttpClient.request/3 method.\n\nFor example, let's call parity's personal_listAccounts method.\n\n```elixir\niex> Ethereumex.HttpClient.request(\"personal_listAccounts\", [], [])\n{:ok,\n [\"0x71cf0b576a95c347078ec2339303d13024a26910\",\n  \"0x7c12323a4fff6df1a25d38319d5692982f48ec2e\"]}\n```","ref":"Ethereumex.html#module-custom-requests"},{"type":"module","title":"Batch requests - Ethereumex","doc":"To send batch requests use Ethereumex.HttpClient.batch_request/1 or Ethereumex.HttpClient.batch_request/2 method.\n\n```elixir\nrequests = [\n   {:web3_client_version, []},\n   {:net_version, []},\n   {:web3_sha3, [\"0x68656c6c6f20776f726c64\"]}\n ]\n Ethereumex.HttpClient.batch_request(requests)\n {\n   :ok,\n   [\n     {:ok, \"Parity//v1.7.2-beta-9f47909-20170918/x86_64-macos/rustc1.19.0\"},\n     {:ok, \"42\"},\n     {:ok, \"0x47173285a8d7341e5e972fc677286384f802f8ef42a5ec5f03bbfa254cb01fad\"}\n   ]\n }\n```","ref":"Ethereumex.html#module-batch-requests"},{"type":"module","title":"Ethereumex.Application","doc":"Configures and starts the :ethereumex OTP application","ref":"Ethereumex.Application.html"},{"type":"function","title":"Ethereumex.Application.start/2","doc":"","ref":"Ethereumex.Application.html#start/2"},{"type":"module","title":"Ethereumex.Client.BaseClient","doc":"The Base Client exposes the Ethereum Client RPC functionality.\n\nWe use a macro so that exposed functions can be used in different behaviours\n(HTTP or IPC).","ref":"Ethereumex.Client.BaseClient.html"},{"type":"module","title":"Ethereumex.Counter","doc":"An application wide RPC2Ethereum client request counter.","ref":"Ethereumex.Counter.html"},{"type":"function","title":"Ethereumex.Counter.get/1","doc":"","ref":"Ethereumex.Counter.html#get/1"},{"type":"function","title":"Ethereumex.Counter.increment/2","doc":"","ref":"Ethereumex.Counter.html#increment/2"},{"type":"function","title":"Ethereumex.Counter.increment/3","doc":"","ref":"Ethereumex.Counter.html#increment/3"},{"type":"function","title":"Ethereumex.Counter.setup/0","doc":"","ref":"Ethereumex.Counter.html#setup/0"},{"type":"module","title":"Ethereumex.WebsocketClient","doc":"WebSocket-based Ethereum JSON-RPC client implementation with real-time subscription support.\n\nThis module provides a WebSocket client interface for both standard Ethereum JSON-RPC calls\nand real-time event subscriptions. It:\n1. Inherits the standard JSON-RPC method definitions from BaseClient\n2. Implements request handling through a persistent WebSocket connection\n3. Provides subscription management for real-time events","ref":"Ethereumex.WebsocketClient.html"},{"type":"module","title":"Standard RPC Usage - Ethereumex.WebsocketClient","doc":"iex> Ethereumex.WebsocketClient.eth_block_number()\n    {:ok, \"0x1234\"}\n\n    iex> Ethereumex.WebsocketClient.eth_get_balance(\"0x407d73d8a49eeb85d32cf465507dd71d507100c1\")\n    {:ok, \"0x0234c8a3397aab58\"}","ref":"Ethereumex.WebsocketClient.html#module-standard-rpc-usage"},{"type":"module","title":"Subscription Usage - Ethereumex.WebsocketClient","doc":"","ref":"Ethereumex.WebsocketClient.html#module-subscription-usage"},{"type":"module","title":"Subscribe to New Blocks - Ethereumex.WebsocketClient","doc":"iex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:newHeads)\n    {:ok, \"0x9cef478923ff08bf67fde6c64013158d\"}\n\n    # Receive notifications in the subscriber process\n    receive do\n      %{\n        \"method\" => \"eth_subscription\",\n        \"params\" => %{\n          \"subscription\" => \"0x9cef478923ff08bf67fde6c64013158d\",\n          \"result\" => %{\"number\" => \"0x1b4\", ...}\n        }\n      } -> :ok\n    end","ref":"Ethereumex.WebsocketClient.html#module-subscribe-to-new-blocks"},{"type":"module","title":"Subscribe to Contract Events - Ethereumex.WebsocketClient","doc":"iex> filter = %{\n    ...>   address: \"0x8320fe7702b96808f7bbc0d4a888ed1468216cfd\",\n    ...>   topics: [\"0xd78a0cb8bb633d06981248b816e7bd33c2a35a6089241d099fa519e361cab902\"]\n    ...> }\n    iex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:logs, filter)\n    {:ok, \"0x4a8a4c0517381924f9838102c5a4dcb7\"}","ref":"Ethereumex.WebsocketClient.html#module-subscribe-to-contract-events"},{"type":"module","title":"Subscribe to Pending Transactions - Ethereumex.WebsocketClient","doc":"iex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:newPendingTransactions)\n    {:ok, \"0x1234567890abcdef1234567890abcdef\"}","ref":"Ethereumex.WebsocketClient.html#module-subscribe-to-pending-transactions"},{"type":"module","title":"Unsubscribe - Ethereumex.WebsocketClient","doc":"# Single subscription\n    iex> Ethereumex.WebsocketClient.unsubscribe(\"0x9cef478923ff08bf67fde6c64013158d\")\n    {:ok, true}\n\n    # Multiple subscriptions\n    iex> Ethereumex.WebsocketClient.unsubscribe([\n    ...>   \"0x9cef478923ff08bf67fde6c64013158d\",\n    ...>   \"0x4a8a4c0517381924f9838102c5a4dcb7\"\n    ...> ])\n    {:ok, true}","ref":"Ethereumex.WebsocketClient.html#module-unsubscribe"},{"type":"module","title":"Features - Ethereumex.WebsocketClient","doc":"* All standard JSON-RPC methods from `Ethereumex.Client.BaseClient`\n* Real-time event subscriptions:\n  - New block headers (`:newHeads`)\n  - Log events (`:logs`)\n  - Pending transactions (`:newPendingTransactions`)\n* Automatic WebSocket connection management\n* Request-response matching\n* Automatic reconnection on failures\n\nThe client maintains a persistent WebSocket connection through `WebsocketServer`,\nwhich handles connection management, request-response matching, subscription\nmanagement, and automatic reconnection on failures.","ref":"Ethereumex.WebsocketClient.html#module-features"},{"type":"type","title":"Ethereumex.WebsocketClient.error/0","doc":"","ref":"Ethereumex.WebsocketClient.html#t:error/0"},{"type":"function","title":"Ethereumex.WebsocketClient.format_batch/1","doc":"","ref":"Ethereumex.WebsocketClient.html#format_batch/1"},{"type":"function","title":"Ethereumex.WebsocketClient.post_request/2","doc":"","ref":"Ethereumex.WebsocketClient.html#post_request/2"},{"type":"function","title":"Ethereumex.WebsocketClient.subscribe/2","doc":"Subscribes to Ethereum events.","ref":"Ethereumex.WebsocketClient.html#subscribe/2"},{"type":"function","title":"Parameters - Ethereumex.WebsocketClient.subscribe/2","doc":"- event_type: Type of event to subscribe to (:newHeads, :logs, :newPendingTransactions)\n  - filter_params: Optional parameters for filtering events (mainly used for :logs)","ref":"Ethereumex.WebsocketClient.html#subscribe/2-parameters"},{"type":"function","title":"Examples - Ethereumex.WebsocketClient.subscribe/2","doc":"# Subscribe to new blocks\n    iex> subscribe(:newHeads)\n    {:ok, \"0x9cef478923ff08bf67fde6c64013158d\"}\n\n    # Subscribe to specific logs\n    iex> subscribe(:logs, %{address: \"0x8320fe7702b96808f7bbc0d4a888ed1468216cfd\"})\n    {:ok, \"0x4a8a4c0517381924f9838102c5a4dcb7\"}\n\nReturns `{:ok, subscription_id}` on success or `{:error, reason}` on failure.","ref":"Ethereumex.WebsocketClient.html#subscribe/2-examples"},{"type":"function","title":"Ethereumex.WebsocketClient.unsubscribe/1","doc":"Unsubscribes from one or more existing subscriptions.","ref":"Ethereumex.WebsocketClient.html#unsubscribe/1"},{"type":"function","title":"Parameters - Ethereumex.WebsocketClient.unsubscribe/1","doc":"- subscription_ids: A single subscription ID or a list of subscription IDs returned from subscribe/2","ref":"Ethereumex.WebsocketClient.html#unsubscribe/1-parameters"},{"type":"function","title":"Examples - Ethereumex.WebsocketClient.unsubscribe/1","doc":"# Unsubscribe from a single subscription\n    iex> unsubscribe(\"0x9cef478923ff08bf67fde6c64013158d\")\n    {:ok, true}\n\n    # Unsubscribe from multiple subscriptions at once\n    iex> unsubscribe([\"0x9cef478923ff08bf67fde6c64013158d\", \"0x4a8a4c0517381924f9838102c5a4dcb7\"])\n    {:ok, true}\n\nReturns `{:ok, true}` on success or `{:error, reason}` on failure.","ref":"Ethereumex.WebsocketClient.html#unsubscribe/1-examples"},{"type":"module","title":"Ethereumex.WebsocketServer","doc":"WebSocket client implementation for Ethereum JSON-RPC API.\n\nThis module manages a persistent WebSocket connection to an Ethereum node and handles\nthe complete request-response cycle for JSON-RPC calls, including subscriptions. It maintains \nstate of ongoing requests, subscriptions, and matches responses to their original callers.","ref":"Ethereumex.WebsocketServer.html"},{"type":"module","title":"Features - Ethereumex.WebsocketServer","doc":"* Standard JSON-RPC requests via WebSocket\n* Real-time event subscriptions (newHeads, logs, newPendingTransactions)\n* Automatic reconnection with exponential backoff\n* Batch request support\n* Concurrent request handling","ref":"Ethereumex.WebsocketServer.html#module-features"},{"type":"module","title":"Request Types - Ethereumex.WebsocketServer","doc":"","ref":"Ethereumex.WebsocketServer.html#module-request-types"},{"type":"module","title":"Standard Requests - Ethereumex.WebsocketServer","doc":"Standard JSON-RPC requests are handled through the `post/1` function:\n\n```elixir\n{:ok, result} = WebsocketServer.post(encoded_request)\n```","ref":"Ethereumex.WebsocketServer.html#module-standard-requests"},{"type":"module","title":"Subscriptions - Ethereumex.WebsocketServer","doc":"The module supports Ethereum's pub/sub functionality for real-time events:\n\n```elixir\n# Subscribe to new block headers\n{:ok, subscription_id} = WebsocketServer.subscribe(%{\n  jsonrpc: \"2.0\",\n  method: \"eth_subscribe\",\n  params: [\"newHeads\"],\n  id: 1\n})\n\n# Subscribe to logs with filter\n{:ok, subscription_id} = WebsocketServer.subscribe(%{\n  jsonrpc: \"2.0\",\n  method: \"eth_subscribe\",\n  params: [\"logs\", %{address: \"0x123...\"}],\n  id: 2\n})\n\n# Unsubscribe (single or multiple subscriptions)\n{:ok, true} = WebsocketServer.unsubscribe(%{\n  jsonrpc: \"2.0\",\n  method: \"eth_unsubscribe\",\n  params: [subscription_id],\n  id: 3\n})\n```","ref":"Ethereumex.WebsocketServer.html#module-subscriptions"},{"type":"module","title":"State Management - Ethereumex.WebsocketServer","doc":"The module maintains several state maps:\n\n```elixir\n%State{\n  requests: %{request_id => caller_pid},                    # Standard requests\n  subscription_requests: %{request_id => caller_pid},       # Pending subscriptions\n  unsubscription_requests: %{request_id => caller_pid},     # Pending unsubscriptions\n  subscriptions: %{subscription_id => subscriber_pid}       # Active subscriptions\n}\n```","ref":"Ethereumex.WebsocketServer.html#module-state-management"},{"type":"module","title":"Subscription Notifications - Ethereumex.WebsocketServer","doc":"When a subscribed event occurs, the notification is automatically forwarded to the\nsubscriber process. Notifications are received as messages in the format:\n\n```elixir\n# New headers notification\n%{\n  \"jsonrpc\" => \"2.0\",\n  \"method\" => \"eth_subscription\",\n  \"params\" => %{\n    \"subscription\" => \"0x9cef478923ff08bf67fde6c64013158d\",\n    \"result\" => %{\n      \"number\" => \"0x1b4\",\n      \"hash\" => \"0x8216c5785ac562ff41e2dcfdf5785ac562ff41e2dcfdf829c5a142f1fccd7d\",\n      \"parentHash\" => \"0x9646252be9520f6e71339a8df9c55e4d7619deeb018d2a3f2d21fc165dde5eb5\"\n    }\n  }\n}\n\n# Logs notification\n%{\n  \"jsonrpc\" => \"2.0\",\n  \"method\" => \"eth_subscription\",\n  \"params\" => %{\n    \"subscription\" => \"0x4a8a4c0517381924f9838102c5a4dcb7\",\n    \"result\" => %{\n      \"address\" => \"0x8320fe7702b96808f7bbc0d4a888ed1468216cfd\",\n      \"topics\" => [\"0xd78a0cb8bb633d06981248b816e7bd33c2a35a6089241d099fa519e361cab902\"],\n      \"data\" => \"0x000000000000000000000000000000000000000000000000000000000000000a\"\n    }\n  }\n}\n```","ref":"Ethereumex.WebsocketServer.html#module-subscription-notifications"},{"type":"module","title":"Error Handling - Ethereumex.WebsocketServer","doc":"- Connection failures are automatically retried with exponential backoff:\n  * Up to `@max_reconnect_attempts` attempts\n  * Starting with `@backoff_initial_delay` ms delay, doubling each attempt\n  * Reconnection attempts are logged\n- Request timeouts after `@request_timeout` ms\n- Invalid JSON responses are handled gracefully\n- Unmatched responses (no waiting caller) are safely ignored\n- Subscription errors are propagated to the subscriber","ref":"Ethereumex.WebsocketServer.html#module-error-handling"},{"type":"type","title":"Ethereumex.WebsocketServer.event_type/0","doc":"","ref":"Ethereumex.WebsocketServer.html#t:event_type/0"},{"type":"function","title":"Ethereumex.WebsocketServer.post/1","doc":"Sends a JSON-RPC request and waits for response.\n\nReturns `{:ok, result}` on success or `{:error, reason}` on failure.\nTimes out after 5000ms.","ref":"Ethereumex.WebsocketServer.html#post/1"},{"type":"type","title":"Ethereumex.WebsocketServer.request_id/0","doc":"","ref":"Ethereumex.WebsocketServer.html#t:request_id/0"},{"type":"function","title":"Ethereumex.WebsocketServer.start_link/1","doc":"Starts the WebSocket connection.","ref":"Ethereumex.WebsocketServer.html#start_link/1"},{"type":"function","title":"Options - Ethereumex.WebsocketServer.start_link/1","doc":"- :url - WebSocket endpoint URL (defaults to Config.websocket_url())\n  - :name - Process name (defaults to __MODULE__)","ref":"Ethereumex.WebsocketServer.html#start_link/1-options"},{"type":"function","title":"Ethereumex.WebsocketServer.subscribe/1","doc":"Subscribes to Ethereum events via WebSocket.\n\nThe request should be a map containing:\n- id: A unique request identifier\n- method: \"eth_subscribe\"\n- params: Parameters for the subscription, including the event type\n\nReturns `{:ok, subscription_id}` on success or `{:error, reason}` on failure.\nTimes out after 5000ms.","ref":"Ethereumex.WebsocketServer.html#subscribe/1"},{"type":"type","title":"Ethereumex.WebsocketServer.subscription_id/0","doc":"","ref":"Ethereumex.WebsocketServer.html#t:subscription_id/0"},{"type":"function","title":"Ethereumex.WebsocketServer.unsubscribe/1","doc":"Unsubscribes from an existing Ethereum event subscription.\n\nThe request should be a map containing:\n- id: A unique request identifier\n- method: \"eth_unsubscribe\"\n- params: A list containing the subscription IDs to unsubscribe from\n\nReturns `{:ok, true}` on success or `{:error, reason}` on failure.\nTimes out after 5000ms.","ref":"Ethereumex.WebsocketServer.html#unsubscribe/1"},{"type":"module","title":"Ethereumex.WebsocketServer.State","doc":"Server state containing:\n- url: WebSocket endpoint URL\n- requests: Map of pending requests with their corresponding sender PIDs\n- subscription requests: Map of pending subscription requests with their corresponding sender PIDs\n- unsubscription requests: Map of pending unsubscription requests with their corresponding sender PIDs\n- subscriptions: Map of subscription IDs to subscriber PIDs","ref":"Ethereumex.WebsocketServer.State.html"},{"type":"type","title":"Ethereumex.WebsocketServer.State.t/0","doc":"","ref":"Ethereumex.WebsocketServer.State.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.11.1 - 2025-02-27 - Changelog","doc":"* Add WebSocket subscription support (https://github.com/mana-ethereum/ethereumex/pull/167)","ref":"changelog.html#0-11-1-2025-02-27"},{"type":"extras","title":"0.11.0 - 2025-02-19 - Changelog","doc":"* WebSocket Client Implementation (https://github.com/mana-ethereum/ethereumex/pull/165)","ref":"changelog.html#0-11-0-2025-02-19"},{"type":"extras","title":"0.10.7 - 2025-01-10 - Changelog","doc":"* Add eth_blob_base_fee support (https://github.com/mana-ethereum/ethereumex/pull/160)","ref":"changelog.html#0-10-7-2025-01-10"},{"type":"extras","title":"0.10.6 - 2023-12-12 - Changelog","doc":"* Add eth_chainId RPC function (https://github.com/mana-ethereum/ethereumex/pull/155)\n* Add json header by default (https://github.com/mana-ethereum/ethereumex/pull/156)","ref":"changelog.html#0-10-6-2023-12-12"},{"type":"extras","title":"0.10.5 - 2023-09-06 - Changelog","doc":"* Update deps (https://github.com/mana-ethereum/ethereumex/pull/153)","ref":"changelog.html#0-10-5-2023-09-06"},{"type":"extras","title":"0.10.4 - 2022-12-07 - Changelog","doc":"* Update finch to 0.14 (https://github.com/mana-ethereum/ethereumex/pull/141)","ref":"changelog.html#0-10-4-2022-12-07"},{"type":"extras","title":"0.10.3 - 2022-09-15 - Changelog","doc":"* Update jason to 1.4 (https://github.com/mana-ethereum/ethereumex/pull/137)","ref":"changelog.html#0-10-3-2022-09-15"},{"type":"extras","title":"0.10.2 - 2022-08-02 - Changelog","doc":"* Fix typespec in HttpClient (https://github.com/mana-ethereum/ethereumex/pull/134)","ref":"changelog.html#0-10-2-2022-08-02"},{"type":"extras","title":"0.10.1 - 2022-07-29 - Changelog","doc":"* Add option to not format batch requests (https://github.com/mana-ethereum/ethereumex/pull/131)","ref":"changelog.html#0-10-1-2022-07-29"},{"type":"extras","title":"0.10.0 - 2022-05-10 - Changelog","doc":"* Add EIP1559 support for eth_maxPriorityFeePerGas and eth_feeHistory (https://github.com/mana-ethereum/ethereumex/pull/127)\n* Update finch to 0.12.0 (https://github.com/mana-ethereum/ethereumex/pull/126)","ref":"changelog.html#0-10-0-2022-05-10"},{"type":"extras","title":"0.9.2 - 2022-04-01 - Changelog","doc":"* Update `finch` to `0.11` (https://github.com/mana-ethereum/ethereumex/commit/28fcb106c04cfc26b78cfbc558acc2385a816ebe)","ref":"changelog.html#0-9-2-2022-04-01"},{"type":"extras","title":"0.9.1 - 2022-02-12 - Changelog","doc":"* Namespace finch adapter under Ethereumex (https://github.com/mana-ethereum/ethereumex/pull/113)\n* Move application into a separate module (https://github.com/mana-ethereum/ethereumex/pull/108)\n* Add http_headers as optional param (https://github.com/mana-ethereum/ethereumex/pull/111)","ref":"changelog.html#0-9-1-2022-02-12"},{"type":"extras","title":"0.9.0 - 2021-12-25 - Changelog","doc":"* Allow adding custom headers (https://github.com/mana-ethereum/ethereumex/pull/97)\n* Handle errors in batch requests (https://github.com/mana-ethereum/ethereumex/pull/98)","ref":"changelog.html#0-9-0-2021-12-25"},{"type":"extras","title":"0.8.0 - 2021-11-21 - Changelog","doc":"* Switch from `httpoison` to `finch` (https://github.com/mana-ethereum/ethereumex/pull/94)","ref":"changelog.html#0-8-0-2021-11-21"},{"type":"extras","title":"0.7.1 - 2021-10-10 - Changelog","doc":"* Change telemetry version (https://github.com/mana-ethereum/ethereumex/pull/92)","ref":"changelog.html#0-7-1-2021-10-10"},{"type":"extras","title":"0.7.0 - 2020-12-07 - Changelog","doc":"* Remove unremovable default hackney pool (https://github.com/mana-ethereum/ethereumex/pull/82)\n* Change ipc_path to absolute path (https://github.com/mana-ethereum/ethereumex/pull/87)\n* Bump deps (https://github.com/mana-ethereum/ethereumex/pull/88)\n* Config.setup_children can inject client type (https://github.com/mana-ethereum/ethereumex/pull/83)\n* Improve typespecs (https://github.com/mana-ethereum/ethereumex/pull/80, https://github.com/mana-ethereum/ethereumex/pull/85, https://github.com/mana-ethereum/ethereumex/pull/86)","ref":"changelog.html#0-7-0-2020-12-07"},{"type":"extras","title":"0.6.4 - 2020-07-24 - Changelog","doc":"* Fix request id exhaustion due to exponential increment (https://github.com/mana-ethereum/ethereumex/pull/76)","ref":"changelog.html#0-6-4-2020-07-24"},{"type":"extras","title":"0.6.3 - 2020-06-30 - Changelog","doc":"* Make batch requests support configurable url via opts argument (https://github.com/mana-ethereum/ethereumex/pull/77)","ref":"changelog.html#0-6-3-2020-06-30"},{"type":"extras","title":"0.6.2 - 2020-05-27 - Changelog","doc":"* Make `telemetry` required dependency (https://github.com/mana-ethereum/ethereumex/pull/73)","ref":"changelog.html#0-6-2-2020-05-27"},{"type":"extras","title":"0.6.1 - 2020-04-16 - Changelog","doc":"* Dependency updates: :httpoison 1.4 -> 1.6, :jason 1.1 -> 1.2, :credo 0.10.2 -> 1.3, :ex_doc 0.19 -> 0.21, :dialyxir 1.0.0-rc.7 -> 1.0.0. And small refactors according to Credo. https://github.com/mana-ethereum/ethereumex/pull/69","ref":"changelog.html#0-6-1-2020-04-16"},{"type":"extras","title":"0.6.0 - 2020-03-02 - Changelog","doc":"* Deprecate measurements via adapter for :telemetry. https://github.com/mana-ethereum/ethereumex/pull/68","ref":"changelog.html#0-6-0-2020-03-02"},{"type":"extras","title":"0.5.6 - 2020-02-28 - Changelog","doc":"* Feature that allows measuring number of RPC calls via an adapter https://github.com/mana-ethereum/ethereumex/pull/67","ref":"changelog.html#0-5-6-2020-02-28"},{"type":"extras","title":"0.5.5 - 2020-10-21 - Changelog","doc":"* Allow request counter resetting (https://github.com/mana-ethereum/ethereumex/pull/65)","ref":"changelog.html#0-5-5-2020-10-21"},{"type":"extras","title":"0.5.4 - 2020-05-23 - Changelog","doc":"* Reported issue with dialyzer specs for boolean arguments in params (https://github.com/mana-ethereum/ethereumex/pull/61)","ref":"changelog.html#0-5-4-2020-05-23"},{"type":"extras","title":"0.5.3 - 2019-02-10 - Changelog","doc":"* Add special case for empty response (https://github.com/mana-ethereum/ethereumex/pull/59)","ref":"changelog.html#0-5-3-2019-02-10"},{"type":"extras","title":"0.5.2 - 2018-12-29 - Changelog","doc":"* Fix `eth_estimateGas` (https://github.com/mana-ethereum/ethereumex/pull/55)\n* Add `eth_getProof` (https://github.com/mana-ethereum/ethereumex/pull/57)","ref":"changelog.html#0-5-2-2018-12-29"},{"type":"extras","title":"0.5.1 - 2018-11-04 - Changelog","doc":"* Replaced poison with jason (https://github.com/mana-ethereum/ethereumex/pull/50)\n* Upgraded httpoison to v1.4 (https://github.com/mana-ethereum/ethereumex/pull/50)","ref":"changelog.html#0-5-1-2018-11-04"},{"type":"extras","title":"0.5.0 - 2018-10-25 - Changelog","doc":"* Remove tunneling requests (https://github.com/exthereum/ethereumex/pull/46)\n* Use poolboy for IpcClient (https://github.com/exthereum/ethereumex/pull/47)","ref":"changelog.html#0-5-0-2018-10-25"},{"type":"extras","title":"0.4.0 - 2018-10-27 - Changelog","doc":"* Use IPC with IpcClient. Choose client type based on config :client_type (https://github.com/exthereum/ethereumex/pull/40)\n* Update poison, credo, dialyzer (https://github.com/exthereum/ethereumex/pull/42)","ref":"changelog.html#0-4-0-2018-10-27"},{"type":"extras","title":"0.3.4 - 2018-09-25 - Changelog","doc":"* Allow configuring GenServer timeout for requests (https://github.com/exthereum/ethereumex/pull/39)","ref":"changelog.html#0-3-4-2018-09-25"},{"type":"extras","title":"0.3.3 - 2018-07-31 - Changelog","doc":"* Added dynamic url input(https://github.com/exthereum/ethereumex/pull/37)","ref":"changelog.html#0-3-3-2018-07-31"},{"type":"extras","title":"0.3.2 - 2018-03-29 - Changelog","doc":"* Fix eth_getLogs mathod params (https://github.com/exthereum/ethereumex/pull/20)","ref":"changelog.html#0-3-2-2018-03-29"},{"type":"extras","title":"0.3.1 - 2018-02-09 - Changelog","doc":"* Handle failed HTTP requests more gracefully (https://github.com/exthereum/ethereumex/pull/19)","ref":"changelog.html#0-3-1-2018-02-09"},{"type":"extras","title":"0.3.0 - 2018-01-23 - Changelog","doc":"* Breaking: Use `:url` config variable instead of `:host`, `:port` and `:scheme` (https://github.com/exthereum/ethereumex/pull/13)","ref":"changelog.html#0-3-0-2018-01-23"},{"type":"extras","title":"0.2.0 - 2018-10-16 - Changelog","doc":"* Breaking: explicit parameters (https://github.com/exthereum/ethereumex/pull/10)","ref":"changelog.html#0-2-0-2018-10-16"},{"type":"extras","title":"0.1.2 - 2017-09-25 - Changelog","doc":"* Added generic request method (https://github.com/exthereum/ethereumex/pull/4)","ref":"changelog.html#0-1-2-2017-09-25"},{"type":"extras","title":"0.1.1 - 2017-09-16 - Changelog","doc":"* Added necessary JSON header for parity (https://github.com/exthereum/ethereumex/pull/2)","ref":"changelog.html#0-1-1-2017-09-16"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright (c) 2018 Ayrat Badykov\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Ethereumex\n\n[![Module Version](https://img.shields.io/hexpm/v/ethereumex.svg)](https://hex.pm/packages/ethereumex)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ethereumex/)\n[![Total Download](https://img.shields.io/hexpm/dt/ethereumex.svg)](https://hex.pm/packages/ethereumex)\n[![License](https://img.shields.io/hexpm/l/ethereumex.svg)](https://github.com/mana-ethereum/ethereumex/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/mana-ethereum/ethereumex.svg)](https://github.com/mana-ethereum/ethereumex/commits/master)\n\n<!-- MDOC !-->\n\nElixir JSON-RPC client for the Ethereum blockchain.\n\nCheck out the documentation [here](https://hexdocs.pm/ethereumex/Ethereumex.html#content).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"Add `:ethereumex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ethereumex, \"~> 0.11.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Overview","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"HTTP - Overview","doc":"In `config/config.exs`, add Ethereum protocol host params to your config file\n\n```elixir\nconfig :ethereumex,\n  url: \"http://localhost:8545\"\n```\n\nYou can also configure the `HTTP` request timeout for requests sent to the Ethereum JSON-RPC\n(you can also overwrite this configuration in `opts` used when calling the client).\n\n```elixir\nconfig :ethereumex,\n  http_options: [pool_timeout: 5000, receive_timeout: 15_000],\n  http_headers: [{\"Content-Type\", \"application/json\"}]\n```\n\n`:pool_timeout` - This timeout is applied when we check out a connection from the pool. Default value is `5_000`.\n`:receive_timeout` - The maximum time to wait for a response before returning an error. Default value is `15_000`","ref":"readme.html#http"},{"type":"extras","title":"IPC - Overview","doc":"If you want to use IPC you will need to set a few things in your config.\n\nFirst, specify the `:client_type`:\n\n```elixir\nconfig :ethereumex,\n  client_type: :ipc\n```\n\nThis will resolve to `:http` by default.\n\nSecond, specify the `:ipc_path`:\n\n```elixir\nconfig :ethereumex,\n  ipc_path: \"/path/to/ipc\"\n```\n\nThe IPC client type mode opens a pool of connection workers (default is 5 and 2, respectively). You can configure the pool size.\n\n```elixir\nconfig :ethereumex,\n  ipc_worker_size: 5,\n  ipc_max_worker_overflow: 2,\n  ipc_request_timeout: 60_000\n```","ref":"readme.html#ipc"},{"type":"extras","title":"WebSocket - Overview","doc":"The WebSocket client supports both standard JSON-RPC requests and real-time subscriptions.\nTo use it, configure your application with:\n\n```elixir\nconfig :ethereumex,\n  websocket_url: \"ws://localhost:8545\",\n  client_type: :websocket\n```\n\n#### Standard RPC Calls\n\nAll standard RPC methods work the same as with HTTP:\n\n```elixir\niex> Ethereumex.WebsocketClient.eth_block_number()\n{:ok, \"0x1234\"}\n```\n\n#### Real-time Subscriptions\n\nSubscribe to various blockchain events:\n\n```elixir\n# Subscribe to new block headers\niex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:newHeads)\n{:ok, \"0x9cef478923ff08bf67fde6c64013158d\"}\n\n# Subscribe to logs/events from specific contracts\niex> filter = %{\n...>   address: \"0x8320fe7702b96808f7bbc0d4a888ed1468216cfd\",\n...>   topics: [\"0xd78a0cb8bb633d06981248b816e7bd33c2a35a6089241d099fa519e361cab902\"]\n...> }\niex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:logs, filter)\n{:ok, \"0x4a8a4c0517381924f9838102c5a4dcb7\"}\n\n# Subscribe to pending transactions\niex> {:ok, subscription_id} = Ethereumex.WebsocketClient.subscribe(:newPendingTransactions)\n{:ok, \"0x1234567890abcdef1234567890abcdef\"}\n\n# Receive notifications in your process\nreceive do\n  %{\n    \"method\" => \"eth_subscription\",\n    \"params\" => %{\n      \"subscription\" => subscription_id,\n      \"result\" => result\n    }\n  } -> handle_notification(result)\nend\n\n# Unsubscribe when done\niex> Ethereumex.WebsocketClient.unsubscribe(subscription_id)\n{:ok, true}\n```\n\nAvailable subscription types:\n\n- `:newHeads` - New block headers\n- `:logs` - Contract events/logs with optional filtering\n- `:newPendingTransactions` - Pending transaction hashes","ref":"readme.html#websocket"},{"type":"extras","title":"Telemetry - Overview","doc":"If you want to count the number of RPC calls per RPC method or overall,\nyou can attach yourself to executed telemetry events.\nThere are two events you can attach yourself to:\n`[:ethereumex]` # has RPC method name in metadata\nEmitted event: `{:event, [:ethereumex], %{counter: 1}, %{method_name: \"method_name\"}}`\n\nor more granular\n`[:ethereumex,  ]` # %{} metadata\nEmitted event: `{:event, [:ethereumex, :method_name_as_atom], %{counter: 1}, %{}}`\n\nEach event caries a single ticker that you can pass into your counters (like `Statix.increment/2`).\nBe sure to add :telemetry as project dependency.","ref":"readme.html#telemetry"},{"type":"extras","title":"Test - Overview","doc":"Download `parity` and initialize the password file\n\n```\n$ make setup\n```\n\nRun `parity`\n\n```\n$ make run\n```\n\nRun tests\n\n```\n$ make test\n```","ref":"readme.html#test"},{"type":"extras","title":"Usage - Overview","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Available methods: - Overview","doc":"- [`web3_clientVersion`](https://eth.wiki/json-rpc/API#web3_clientversion)\n- [`web3_sha3`](https://eth.wiki/json-rpc/API#web3_sha3)\n- [`net_version`](https://eth.wiki/json-rpc/API#net_version)\n- [`net_peerCount`](https://eth.wiki/json-rpc/API#net_peercount)\n- [`net_listening`](https://eth.wiki/json-rpc/API#net_listening)\n- [`eth_protocolVersion`](https://eth.wiki/json-rpc/API#eth_protocolversion)\n- [`eth_syncing`](https://eth.wiki/json-rpc/API#eth_syncing)\n- [`eth_coinbase`](https://eth.wiki/json-rpc/API#eth_coinbase)\n- [`eth_chainId`](https://eth.wiki/json-rpc/API#eth_chainId)\n- [`eth_mining`](https://eth.wiki/json-rpc/API#eth_mining)\n- [`eth_hashrate`](https://eth.wiki/json-rpc/API#eth_hashrate)\n- [`eth_gasPrice`](https://eth.wiki/json-rpc/API#eth_gasprice)\n- [`eth_accounts`](https://eth.wiki/json-rpc/API#eth_accounts)\n- [`eth_blockNumber`](https://eth.wiki/json-rpc/API#eth_blocknumber)\n- [`eth_getBalance`](https://eth.wiki/json-rpc/API#eth_getbalance)\n- [`eth_getStorageAt`](https://eth.wiki/json-rpc/API#eth_getstorageat)\n- [`eth_getTransactionCount`](https://eth.wiki/json-rpc/API#eth_gettransactioncount)\n- [`eth_getBlockTransactionCountByHash`](https://eth.wiki/json-rpc/API#eth_getblocktransactioncountbyhash)\n- [`eth_getBlockTransactionCountByNumber`](https://eth.wiki/json-rpc/API#eth_getblocktransactioncountbynumber)\n- [`eth_getUncleCountByBlockHash`](https://eth.wiki/json-rpc/API#eth_getunclecountbyblockhash)\n- [`eth_getUncleCountByBlockNumber`](https://eth.wiki/json-rpc/API#eth_getunclecountbyblocknumber)\n- [`eth_getCode`](https://eth.wiki/json-rpc/API#eth_getcode)\n- [`eth_sign`](https://eth.wiki/json-rpc/API#eth_sign)\n- [`eth_sendTransaction`](https://eth.wiki/json-rpc/API#eth_sendtransaction)\n- [`eth_sendRawTransaction`](https://eth.wiki/json-rpc/API#eth_sendrawtransaction)\n- [`eth_call`](https://eth.wiki/json-rpc/API#eth_call)\n- [`eth_estimateGas`](https://eth.wiki/json-rpc/API#eth_estimategas)\n- [`eth_getBlockByHash`](https://eth.wiki/json-rpc/API#eth_getblockbyhash)\n- [`eth_getBlockByNumber`](https://eth.wiki/json-rpc/API#eth_getblockbynumber)\n- [`eth_getTransactionByHash`](https://eth.wiki/json-rpc/API#eth_gettransactionbyhash)\n- [`eth_getTransactionByBlockHashAndIndex`](https://eth.wiki/json-rpc/API#eth_gettransactionbyblockhashandindex)\n- [`eth_getTransactionByBlockNumberAndIndex`](https://eth.wiki/json-rpc/API#eth_gettransactionbyblocknumberandindex)\n- [`eth_getTransactionReceipt`](https://eth.wiki/json-rpc/API#eth_gettransactionreceipt)\n- [`eth_getUncleByBlockHashAndIndex`](https://eth.wiki/json-rpc/API#eth_getunclebyblockhashandindex)\n- [`eth_getUncleByBlockNumberAndIndex`](https://eth.wiki/json-rpc/API#eth_getunclebyblocknumberandindex)\n- [`eth_getCompilers`](https://eth.wiki/json-rpc/API#eth_getcompilers)\n- [`eth_compileLLL`](https://eth.wiki/json-rpc/API#eth_compilelll)\n- [`eth_compileSolidity`](https://eth.wiki/json-rpc/API#eth_compilesolidity)\n- [`eth_compileSerpent`](https://eth.wiki/json-rpc/API#eth_compileserpent)\n- [`eth_newFilter`](https://eth.wiki/json-rpc/API#eth_newfilter)\n- [`eth_newBlockFilter`](https://eth.wiki/json-rpc/API#eth_newblockfilter)\n- [`eth_newPendingTransactionFilter`](https://eth.wiki/json-rpc/API#eth_newpendingtransactionfilter)\n- [`eth_uninstallFilter`](https://eth.wiki/json-rpc/API#eth_uninstallfilter)\n- [`eth_getFilterChanges`](https://eth.wiki/json-rpc/API#eth_getfilterchanges)\n- [`eth_getFilterLogs`](https://eth.wiki/json-rpc/API#eth_getfilterlogs)\n- [`eth_getLogs`](https://eth.wiki/json-rpc/API#eth_getlogs)\n- eth_getProof\n- [`eth_getWork`](https://eth.wiki/json-rpc/API#eth_getwork)\n- [`eth_submitWork`](https://eth.wiki/json-rpc/API#eth_submitwork)\n- [`eth_submitHashrate`](https://eth.wiki/json-rpc/API#eth_submithashrate)\n- [`db_putString`](https://eth.wiki/json-rpc/API#db_putstring)\n- [`db_getString`](https://eth.wiki/json-rpc/API#db_getstring)\n- [`db_putHex`](https://eth.wiki/json-rpc/API#db_puthex)\n- [`db_getHex`](https://eth.wiki/json-rpc/API#db_gethex)\n- [`shh_post`](https://eth.wiki/json-rpc/API#shh_post)\n- [`shh_version`](https://eth.wiki/json-rpc/API#shh_version)\n- [`shh_newIdentity`](https://eth.wiki/json-rpc/API#shh_newidentity)\n- [`shh_hasIdentity`](https://eth.wiki/json-rpc/API#shh_hasidentity)\n- [`shh_newGroup`](https://eth.wiki/json-rpc/API#shh_newgroup)\n- [`shh_addToGroup`](https://eth.wiki/json-rpc/API#shh_addtogroup)\n- [`shh_newFilter`](https://eth.wiki/json-rpc/API#shh_newfilter)\n- [`shh_uninstallFilter`](https://eth.wiki/json-rpc/API#shh_uninstallfilter)\n- [`shh_getFilterChanges`](https://eth.wiki/json-rpc/API#shh_getfilterchanges)\n- [`shh_getMessages`](https://eth.wiki/json-rpc/API#shh_getmessages)\n\n#### WebSocket Subscription Methods\n\n- [`eth_subscribe`](https://geth.ethereum.org/docs/interacting-with-geth/rpc/pubsub#create-subscriptions) - Subscribe to real-time events\n- [`eth_unsubscribe`](https://geth.ethereum.org/docs/interacting-with-geth/rpc/pubsub#cancel-subscriptions) - Unsubscribe from events","ref":"readme.html#available-methods"},{"type":"extras","title":"IpcClient - Overview","doc":"You can follow along with any of these examples using IPC by replacing `HttpClient` with `IpcClient`.","ref":"readme.html#ipcclient"},{"type":"extras","title":"Examples - Overview","doc":"```elixir\niex> Ethereumex.HttpClient.web3_client_version\n{:ok, \"Parity//v1.7.2-beta-9f47909-20170918/x86_64-macos/rustc1.19.0\"}\n\n# Using the url option will overwrite the configuration\niex> Ethereumex.HttpClient.web3_client_version(url: \"http://localhost:8545\")\n{:ok, \"Parity//v1.7.2-beta-9f47909-20170918/x86_64-macos/rustc1.19.0\"}\n\niex> Ethereumex.HttpClient.web3_sha3(\"wrong_param\")\n{:error, %{\"code\" => -32602, \"message\" => \"Invalid params: invalid format.\"}}\n\niex> Ethereumex.HttpClient.eth_get_balance(\"0x407d73d8a49eeb85d32cf465507dd71d507100c1\")\n{:ok, \"0x0\"}\n```\n\nNote that all method names are snakecases, so, for example, shh_getMessages method has corresponding Ethereumex.HttpClient.shh_get_messages/1 method. Signatures can be found in Ethereumex.Client.Behaviour. There are more examples in tests.\n\n#### eth_call example - Read only smart contract calls\n\nIn order to call a smart contract using the JSON-RPC interface you need to properly hash the data attribute (this will need to include the contract method signature along with arguments if any). You can do this manually or use a hex package like [ABI](https://hex.pm/packages/ex_abi) to parse your smart contract interface or encode individual calls.\n\n```elixir\ndefp deps do\n  [\n    ...\n    {:ethereumex, \"~> 0.9\"},\n    {:ex_abi, \"~> 0.5\"}\n    ...\n  ]\nend\n```\n\nNow load the ABI and pass the method signature. Note that the address needs to be converted to bytes:\n\n```elixir\naddress           = \"0xF742d4cE7713c54dD701AA9e92101aC42D63F895\" |> String.slice(2..-1) |> Base.decode16!(case: :mixed)\ncontract_address  = \"0xC28980830dD8b9c68a45384f5489ccdAF19D53cC\"\nabi_encoded_data  = ABI.encode(\"balanceOf(address)\", [address]) |> Base.encode16(case: :lower)\n```\n\nNow you can use eth_call to execute this smart contract command:\n\n```elixir\nbalance_bytes = Ethereumex.HttpClient.eth_call(%{\n  data: \"0x\" <> abi_encoded_data,\n  to: contract_address\n})\n```\n\nTo convert the balance into an integer:\n\n```elixir\nbalance_bytes\n|> String.slice(2..-1)\n|> Base.decode16!(case: :lower)\n|> TypeDecoder.decode_raw([{:uint, 256}])\n|> List.first\n```","ref":"readme.html#examples"},{"type":"extras","title":"Custom requests - Overview","doc":"Many Ethereum protocol implementations support additional JSON-RPC API methods. To use them, you should call Ethereumex.HttpClient.request/3 method.\n\nFor example, let's call parity's personal_listAccounts method.\n\n```elixir\niex> Ethereumex.HttpClient.request(\"personal_listAccounts\", [], [])\n{:ok,\n [\"0x71cf0b576a95c347078ec2339303d13024a26910\",\n  \"0x7c12323a4fff6df1a25d38319d5692982f48ec2e\"]}\n```","ref":"readme.html#custom-requests"},{"type":"extras","title":"Batch requests - Overview","doc":"To send batch requests use Ethereumex.HttpClient.batch_request/1 or Ethereumex.HttpClient.batch_request/2 method.\n\n```elixir\nrequests = [\n   {:web3_client_version, []},\n   {:net_version, []},\n   {:web3_sha3, [\"0x68656c6c6f20776f726c64\"]}\n ]\n Ethereumex.HttpClient.batch_request(requests)\n {\n   :ok,\n   [\n     {:ok, \"Parity//v1.7.2-beta-9f47909-20170918/x86_64-macos/rustc1.19.0\"},\n     {:ok, \"42\"},\n     {:ok, \"0x47173285a8d7341e5e972fc677286384f802f8ef42a5ec5f03bbfa254cb01fad\"}\n   ]\n }\n```\n\n<!-- MDOC !-->","ref":"readme.html#batch-requests"},{"type":"extras","title":"Built on Ethereumex - Overview","doc":"If you are curious what others are building with ethereumex, you might want to take a look at these projects:\n\n- [ethers](https://github.com/ExWeb3/elixir_ethers) - Interacting with EVM contracts like first-class Elixir functions similar to Ethers.js","ref":"readme.html#built-on-ethereumex"},{"type":"extras","title":"Contributing - Overview","doc":"1. [Fork it!](http://github.com/ayrat555/ethereumex/fork)\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin my-new-feature`)\n5. Create new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2018 Ayrat Badykov\n\nReleased under the MIT License, which can be found in the repository in\n[LICENSE.md](./LICENSE.md).","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}